﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace m10_client
{
    public partial class Form3 : M10ClientForm
    {
         // このフォームが１つだけしか開けないようにする
        // Singleton適用
        private static Form3 _instance;

        public static Form3 Instance
        {
            get
            {
                if (_instance == null)
                     _instance = new Form3();

                return _instance;
            }
        }

        public static void CloseInstance()
        {
            if (_instance != null)
                _instance.Close();
        }

        //設定ファイルから複数サーバのIP+ポートを読み込み
        //Connectionオブジェクトを作成
        MultipleConnectionsController controller;
        
        private Form3()
        {
            InitializeComponent();
        }

        private void Form3_Load(object sender, EventArgs e)
        {
            controller = new MultipleConnectionsController(this);
            if (false == controller.InitFormUI())
                _instance.Close();
        }

        private void SendText_Click(object sender, EventArgs e)
        {
            controller.SendText(txtData.Text, listServers);
            txtData.Focus();
        }

        private void Reset_Click(object sender, EventArgs e)
        {
            controller.SendReset(listServers);
            txtData.Focus();
        }

        //M10.M10ClientFormのメンバー
        // MultipleConnectionsController.InitFormUI(M10.M10ClientForm frm )から呼び出される
        public override void InitFormUI()
        {
            //リストボックスにサーバ名を設定
            for (int i = 0; i < controller.ConnectionCount; i++)
            {
                listServers.Items.Add(string.Format("{0}.{1}", i, controller[i].Name));
            }

            //リストボックス項目すべて選択
            for (int i = 0; i < listServers.Items.Count; i++)
                listServers.SetSelected(i, true);
           
        }

        //M10.M10ClientFormのメンバー
        // MultipleConnectionsControllerの送信メソッドからステータス更新で呼び出される
        public override void SetStatusText(string msg)
        {
            toolStripStatusLabel1.Text = msg;
        }

        public override void SetResultStatusText(string msg)
        {
            toolStripStatusLabel2.Text = msg;
        }


        private void Form3_FormClosed(object sender, FormClosedEventArgs e)
        {
            _instance = null;
        }

    }
}
