﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace m10_client
{
    class M10ClientUtil
    {
        static public string AppName= "M10クライアント";

        static public void ShowError(string strErrMsg)
        {
            MessageBox.Show(strErrMsg,
                            AppName,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Exclamation);
        }
    }

    public class SingleConnectionController : IDisposable
    {
#if SEND_WAIT
       const bool bSendWait=true;
#else
        const bool bSendWait = false;
#endif

        static M10ClientForm _frm;

        M10.Connection conn;

        public M10.Connection CurrentConnection
        {
            get { return conn; }
        }

        public SingleConnectionController(M10ClientForm frm)
        {
            _frm = frm;

            M10.ServerSetting setting = new M10.ServerSetting();

            conn = new M10.Connection(setting.M10Server);
        }

        ~SingleConnectionController()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected void Dispose(bool disposing)
        {
            if (disposing)
                GC.SuppressFinalize(this);

            Close();
        }

        void Close()
        {
            try
            {
                conn.CloseNet();
            }
            catch (Exception ex)
            {
                M10ClientUtil.ShowError(string.Format("サーバとの接続切断 : {01}", ex.Message));
            }
        }

        public bool InitFormUI()
        {
            while (conn.ServerIP == "")
            {
                m10_SetIP.SetIP frmSetIP = new m10_SetIP.SetIP();
                if (DialogResult.OK == frmSetIP.ShowDialog(_frm))
                {
                    conn.Port = frmSetIP.Port;
                    conn.ServerIP = frmSetIP.IP;
                }
                else
                {
                    M10ClientUtil.ShowError(string.Format("サーバのIPをINIファイルで設定してください. 続行できません."));
                    return false;
                }
            }

#if ASYNC_CALL
            conn.OnSendCompleted += new M10.Connection.NotifySendCompletedHandler(conn_OnSendCompleted);
#endif

            return true;
        }

#if ASYNC_CALL
        delegate void SetResultStatusTextDelegate(string msg);

         void conn_OnSendCompleted(object sender, M10.OnSendCompletedEventArgs e)
        {
            if (e.status == false)
                MessageBox.Show(e.connName + "でエラー: " + e.msg, "M10_Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                // _frmがstaticでないとベースクラス M10ClientForm のSetResultStatusTextを呼び出す
                string msg = e.connName + "の送信ID " + e.id + " 成功";
                if (_frm.InvokeRequired == true)
                    _frm.Invoke(new SetResultStatusTextDelegate(_frm.SetResultStatusText), msg);
                else
                    _frm.SetResultStatusText(msg);

                _frm.SetResultStatusText(e.connName + "の送信ID " + e.id + " 成功");
            }
        }
#endif

        private void Send(string DataToSend, bool bCloseOnSent)
        {
             if (conn.Status == false)
            {
                if (conn.errCount > 0)
                {
                    if (DialogResult.No == MessageBox.Show("サーバへの接続は失敗しています。再度接続を試みますか？", M10ClientUtil.AppName,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2))
                    {
                        return;
                    }
                }

                _frm.SetStatusText("サーバに接続中...");
                Application.DoEvents();
            }

            try
            {
                uint id;

                if (DataToSend == "")
                    conn.SendReset();
                else
                {
#if ASYNC_CALL
                    id = conn.SendStringAsync(DataToSend, bSendWait, bCloseOnSent);
#else
                    id = conn.SendString(DataToSend, bSendWait, bCloseOnSent);
#endif
                }
            }
            catch (Exception ex)
            {
                M10ClientUtil.ShowError(ex.Message);
            }
            finally
            {

                _frm.SetStatusText("レディ");
            }
        }

        public void SendReset()
        {
            Send("",false);
        }

        public void SendText(string DataToSend)
        {
            if( DataToSend == "" )
            {   M10ClientUtil.ShowError("送信するキー操作文字列を設定してください");
                return;
            }
            Send(DataToSend,false);
        }

    }

    public class MultipleConnectionsController : IDisposable
    {
#if SEND_WAIT
       const bool bSendWait=true;
#else
        const bool bSendWait = false;
#endif

        static M10ClientForm _frm;

        int connectionCount=0;

        //List<M10.Connection> m10conns = new List<M10.Connection>();
        M10.Connection[] m10conns;

        public int ConnectionCount
        {
            get { return connectionCount; }
        }

        public M10.Connection this[int key]
        {
            get
            {
                if (key >= connectionCount || key < 0)
                    throw new IndexOutOfRangeException();

                return m10conns[key];
            }
        }

        int CreateConnectionObjects()
        {
            M10.Servers svrs = null;

            try
            {
                //設定ファイル読み込み
                svrs = new M10.Servers();

                // XML設定ファイルを利用する
                // XmlServersクラスは Serversクラスから派生
                //svrs = new M10.XmlServers();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, M10ClientUtil.AppName,
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return 0;
            }

            if (svrs.Count == 0)
                return 0;

            //設定されているサーバ数
            connectionCount = svrs.Count;

            //各サーバへの Connectionオブジェクト配列を設定
            m10conns = new M10.Connection[connectionCount];

            for (int i = 0; i < connectionCount; i++)
                m10conns[i] = new M10.Connection(svrs[i]);

            return connectionCount;
        }

        public MultipleConnectionsController(M10ClientForm frm)
        {
            _frm = frm;

            CreateConnectionObjects();
        }

        ~MultipleConnectionsController()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
        }

        protected void Dispose(bool disposing)
        {
            if (disposing == true)
                GC.SuppressFinalize(this);

            Close();
        }

        void Close()
        {
            // 初期化される前に閉じられる可能性あり
            if (m10conns == null)
                return;

            //各サーバとの接続切断
            foreach (M10.Connection conn in m10conns)
            {
                try
                {
                    conn.CloseNet();
                }
                catch (Exception ex)
                {
                    M10ClientUtil.ShowError(string.Format("[Server{0}との接続切断 : {1}", conn.No + 1, ex.Message));
                }
            }
        }

        public bool InitFormUI()
        {
            // Connectionオブジェクトが生成されていない
            while (ConnectionCount == 0)
            {
                m10_SetIP.SetIP frmSetIP = new m10_SetIP.SetIP();
                 if (DialogResult.OK == frmSetIP.ShowDialog(_frm))
                 {
                     if( 0 == CreateConnectionObjects() )
                     {                    
                         connectionCount = 1;

                         m10conns = new M10.Connection[connectionCount];

                         m10conns[0] = new M10.Connection(0, "Server1", frmSetIP.Port, frmSetIP.IP);
                     }

                 }
                 else
                 {
                    M10ClientUtil.ShowError("設定ファイルを確認してください");
                    return false;
                 }
            }

            // ConnectionオブジェクトのIPが設定されていることを確認
            for (int i = 0; i < ConnectionCount; i++)
            {
                if (m10conns[i].ServerIP == "")
                {
                    M10ClientUtil.ShowError(string.Format("[Server{0}]のIPを確認してください", i + 1));
                    return false;
                }
            }

            _frm.InitFormUI();

#if ASYNC_CALL
            // イベントハンドラー登録.　すべての Connectionオブジェクトで同じハンドラを登録
            // ハンドラは OnSendCompletedEventArgsで、どのオブジェクトから送られてきたか判断できる
            foreach( M10.Connection conn in m10conns )
                conn.OnSendCompleted += new M10.Connection.NotifySendCompletedHandler(conn_OnSendCompleted);
#endif

            return true;
        }

#if ASYNC_CALL
        delegate void SetResultStatusTextDelegate(string msg);

        void conn_OnSendCompleted(object sender, M10.OnSendCompletedEventArgs e)
        {
            if (e.status == false)
                MessageBox.Show(e.connName + "でエラー: " + e.msg, "M10_Client", MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                // _frmがstaticでないとベースクラス M10ClientForm のSetResultStatusTextを呼び出す
                string msg = e.connName + "の送信ID " + e.id + " 成功";
                if (_frm.InvokeRequired == true)
                    _frm.Invoke(new SetResultStatusTextDelegate(_frm.SetResultStatusText), msg);
                else
                    _frm.SetResultStatusText(msg);

                _frm.SetResultStatusText(e.connName + "の送信ID " + e.id + " 成功");
            }

        }
#endif

        private void Send(M10.Connection conn, string DataToSend, bool bCloseOnSent)
        {
            if (conn.Status == false)
            {
                if (conn.errCount > 0)
                {
                    if (DialogResult.No == MessageBox.Show(string.Format("[Server{0}]への接続は失敗しています。再度接続を試みますか？", conn.No + 1), M10ClientUtil.AppName,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2))
                    {
                        return;
                    }
                }


                _frm.SetStatusText(string.Format("[Server{0}]に接続中...", conn.No + 1));
                Application.DoEvents();
            }

            _frm.Cursor = Cursors.WaitCursor;

            try
            {
                if (DataToSend == "")
                    conn.SendReset();
                else
                {
#if ASYNC_CALL
                    conn.SendStringAsync(DataToSend, bSendWait, bCloseOnSent);
#else
                    conn.SendString(DataToSend, bSendWait, bCloseOnSent);
#endif
                }
            }
            catch (Exception ex)
            {
                _frm.Cursor = Cursors.Default;
                M10ClientUtil.ShowError(ex.Message);
            }
            finally
            {
                _frm.Cursor = Cursors.Default;
                _frm.SetStatusText("レディ");
            }
        }

        public void SendReset()
        {
            foreach (M10.Connection conn in m10conns)
                Send(conn, "",false);
        }

        public void SendReset(ListBox listServers)
        {
            for (int i = 0; i < listServers.Items.Count; i++)
            {
                //リストボックスで選択されていなければ送信しない
                if (listServers.GetSelected(i) == false)
                    continue;

                Send(m10conns[i], "", true);
            }
        }

        public void SendText( string DataToSend, ListBox listServers)
        {
            for (int i = 0; i < listServers.Items.Count; i++)
            {
                //リストボックスで選択されていなければ送信しない
                if( listServers.GetSelected(i) == false )
                    continue;

                Send(m10conns[i], DataToSend, true);
            }
        }

        public void SendText( string DataToSend)
        {
            foreach (M10.Connection conn in m10conns)
                Send( conn,DataToSend, false);
        }
    }
}
