﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace m10_client_simple
{
    public partial class Form1 : Form
    {
#if SEND_WAIT
        const bool bSendWait = true;
#else
        const bool bSendWait = false;
#endif

        M10.Connection conn = null;

        public Form1()
        {
            InitializeComponent();

            //設定ファイルを読み込む
            M10.ServerSetting setting = new M10.ServerSetting();

            //Connectionオブジェクトを作成
            conn = new M10.Connection(setting.M10Server);

            //設定ファイルを使わずに、直接コードでIP/ポートを指定すると
            //Form1_Loadedの処理はイベントハンドラ登録だけ必要
            //例：
            //conn = new M10.Connection(0, "Server1", 12001, "192.169.0.0");
               
       }

        private void Form1_Load(object sender, EventArgs e)
        {
            // ConnectionオブジェクトのServerIPプロパティが空ならば
            // SetIPフォームをモーダルダイアログで開く
            while (conn.ServerIP == "")
            {
                m10_SetIP.SetIP frmSetIP = new m10_SetIP.SetIP();
                if (DialogResult.OK == frmSetIP.ShowDialog(this))
                {
                    conn.Port = frmSetIP.Port;
                    conn.ServerIP = frmSetIP.IP;
                }
                else
                {
                    M10ClientUtil.ShowError("設定ファイルでIPアドレスを設定してください");
                    Application.Exit();
                    break;
                }
            }
            if (conn.ServerIP == "") return;

#if ASYNC_CALL
            //イベントハンドラー登録
            conn.OnSendCompleted += new M10.Connection.NotifySendCompletedHandler(conn_OnSendCompleted);
#endif
        }

#if ASYNC_CALL
        delegate void SetResultStatusTextDelegate(string msg);

        void SetResultStatusText(string msg)
        {
            toolStripStatusLabel2.Text = msg;
        }

        // フォームスレッドとは別スレッドで呼び出される
        void conn_OnSendCompleted(object sender, M10.OnSendCompletedEventArgs e)
        {
            if (e.status)
            {
                string msg = "送信ID " + e.id + " 成功";

                //現在のスレッドではなくフォームスレッドでラベルにテキストを
                //設定する必要があれば Invokeする
                if (InvokeRequired == true)
                    Invoke(new SetResultStatusTextDelegate(SetResultStatusText), msg);
                else
                    SetResultStatusText(msg);
            }
            else
                MessageBox.Show(e.id + "でエラー : " + e.msg, "M10_Client_Simple", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
#endif

        private void btnSend_Click(object sender, EventArgs e)
        {
            // 文字列送信
            if (textBox1.Text.Length > 0)
                Send(textBox1.Text);
            else
                toolStripStatusLabel1.Text = "文字列を設定してください";

            //　文字列クリア
            // textBox1.Text = "";

            // 文字列をすぐに入力できるように
            textBox1.Focus();
        }

        //
        // このフォームでのメイン処理
        // 送信処理テンプレート
        // 
        private void Send(string DataToSend)
        {
            //サーバーと未接続な状態
            if (conn.Status == false)
            {
                // 接続に失敗している場合、接続不能の可能性あり
                // 念のために確認を促す
                if (conn.errCount > 0)
                {
                    if (DialogResult.No == MessageBox.Show("サーバへの接続は失敗しています。再度接続を試みますか？", M10ClientUtil.AppName,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2))
                    {
                        return;
                    }
                }

                toolStripStatusLabel1.Text = "サーバに接続中...";

                // ラベルが描画されるよう、いったんOSに制御を戻す
                Application.DoEvents();
            }

            //接続は時間がかかるかもしれないため
            //カーソル形状を変更
            Cursor = Cursors.WaitCursor;

            try
            {
                if (DataToSend == "")
                    conn.SendReset();
                else
                {
#if ASYNC_CALL
                    conn.SendStringAsync(DataToSend, bSendWait, false);
#else
                    uint id = conn.SendString(DataToSend, bSendWait, false);
                    toolStripStatusLabel2.Text = "送信ID " + id + " 成功" ;
#endif
                }
            }
            catch (Exception ex)
            {
                Cursor = Cursors.Default;
                M10ClientUtil.ShowError(ex.Message);
            }
            finally
            {
                Cursor = Cursors.Default;
                toolStripStatusLabel1.Text = "レディ";
            }
        }


        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            //Connectionオブジェクトのリソース解放
            conn.Dispose();
        }
    }

    class M10ClientUtil
    {
        static public string AppName = "M10クライアント";

        static public void ShowError(string strErrMsg)
        {
            MessageBox.Show(strErrMsg,
                            AppName,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Exclamation);
        }
    }

}
