﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;
using System.Windows.Threading;

//参照設定で M10LIB.DLL が追加されている

namespace m10_client_simple_wpf
{
    /// <summary>
    /// Window1.xaml の相互作用ロジック
    /// </summary>
    public partial class Window1 : Window
    {
#if SEND_WAIT
        const bool bSendWait = true;
#else
        const bool bSendWait = false;
#endif

        const string AppName = "キー操作文字列送信";

        M10.Connection conn;

        public Window1()
        {
            InitializeComponent();

        }

        private void Window_Initialized(object sender, EventArgs e)
        {
            try
            {
                // 設定ファイル読み込み
                M10.ServerSetting setting = new M10.ServerSetting();

                // 読み込んだ M10Server オブジェクトで Connectionを作成
                conn = new M10.Connection(setting.M10Server);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, AppName,
                                   MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }

            // Connectionオブジェクトの ServerIPプロパティが空（設定ファイルで設定していない、
            //　設定ファイルが空 )ならば、IP/Portの入力を促す
            while (conn.ServerIP == "")
            {
                // SetIPウィンドウをモーダルダイアログで開く
                SetIP setIP = new SetIP();

                if (setIP.ShowDialog() == true)
                {
                    conn.Port = setIP.Port;
                    conn.ServerIP = setIP.IP;
                }
                else
                {
                    MessageBox.Show("サーバのIPを設定ファイルで指定してください", AppName,
                                       MessageBoxButton.OK, MessageBoxImage.Exclamation);

                    // ウィンドウを閉じる
                    Close();
                    break;
                }
            }

#if ASYNC_CALL
            conn.OnSendCompleted += new M10.Connection.NotifySendCompletedHandler(conn_OnSendCompleted);
#endif
        }

#if ASYNC_CALL

        public delegate void UpdateStatusText(string msg);

        // UIスレッドとは別スレッドから呼び出される
        void conn_OnSendCompleted(object sender, M10.OnSendCompletedEventArgs e)
        {
            if (e.status)
            {
                //UIスレッド上のStatusBar1.Items[1]を更新する
                Dispatcher.Invoke(DispatcherPriority.Normal,
                                   new UpdateStatusText(SetStatusText),
                                   "送信ID " + e.id + " 成功"); 
            }
            else
                MessageBox.Show("送信ID " + e.id + "が失敗 : " + e.msg, AppName,
                                    MessageBoxButton.OK, MessageBoxImage.Error);

        }

#endif

        void SetStatusText(string msg)
        {
            StatusBarItem item = (StatusBarItem)statusBar1.Items[1];
            item.Content = msg;
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            if (textBox1.Text.Length > 0)
            {
                Send(textBox1.Text);
                textBox1.Focus();
            }
            else
                ((StatusBarItem)statusBar1.Items[0]).Content = "送信する文字列を設定してください";
        }

        private void btnReset_Click(object sender, RoutedEventArgs e)
        {
            //空文字列で Send を呼び出すとリセットコードを送出
            Send("");

            textBox1.Focus();
        }

        private void Send(string DataToSend)
        {
            StatusBarItem item = (StatusBarItem)statusBar1.Items[0];

            //サーバーと未接続
            if (conn.Status == false)
            {
                // 接続に失敗している場合、接続不能の可能性あり
                // 念のために確認を促す
                if (conn.errCount > 0)
                {
                    if (MessageBoxResult.No == MessageBox.Show("サーバへの接続は失敗しています。再度接続を試みますか？", AppName,
                          MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.No))
                    {
                        return;
                    }
                }

                //接続は時間がかかるかもしれないため
                //カーソル形状を変更
                this.Cursor = Cursors.Wait;

                item.Content = "サーバに接続中...";

                // item.Contentが描画されるよう、いったんOSに制御を戻す
                DoEvents();
            }

            try
            {
                if (DataToSend == "")
                    conn.SendReset();
                else
                {
#if ASYNC_CALL
                    conn.SendStringAsync(DataToSend,bSendWait,false);
#else
                    uint id = conn.SendString(DataToSend, bSendWait, false);
                    SetStatusText("送信ID " + id + " 成功");
#endif
                }
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Arrow;

                MessageBox.Show(ex.Message, AppName, MessageBoxButton.OK,MessageBoxImage.Exclamation);
            }
            finally
            {
                this.Cursor = Cursors.Arrow;
                item.Content = "レディ";
            }
        }

        //
        // WPF での DoEvents()実装
        //
        // http://msdn2.microsoft.com/ja-jp/library/system.windows.threading.dispatcher.pushframe(VS.80).aspx 
        //
        public void DoEvents()
        {
            DispatcherFrame frame = new DispatcherFrame();
            Dispatcher.CurrentDispatcher.BeginInvoke(DispatcherPriority.Background,
                new DispatcherOperationCallback(ExitFrames), frame);
            Dispatcher.PushFrame(frame);
        }

        public object ExitFrames(object f)
        {
            ((DispatcherFrame)f).Continue = false;

            return null;
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            conn.Dispose();
        }


    }
}
